"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const assert_1 = __importDefault(require("assert"));
const uuid_1 = __importDefault(require("uuid"));
const node_eventstore_client_1 = require("node-eventstore-client");
class NestjsEventStore {
    constructor() {
        this.logger = new common_1.Logger(this.constructor.name);
        this.type = 'event-store';
    }
    connect(options, endpoint) {
        try {
            this.connection = node_eventstore_client_1.createConnection(options, endpoint);
            this.connection.connect();
            this.connection.on('connected', () => {
                this.isConnected = true;
                this.logger.log('EventStore connected!');
            });
            this.connection.on('closed', () => {
                this.isConnected = false;
                this.logger.error('EventStore closed!');
                this.connect(options, endpoint);
            });
            return this;
        }
        catch (e) {
            this.logger.error(e.message);
            throw new Error(e);
        }
    }
    getConnection() {
        return this.connection;
    }
    newEvent(name, payload) {
        return this.newEventBuilder(name, payload);
    }
    newEventBuilder(eventType, data, metadata, eventId) {
        assert_1.default(eventType);
        assert_1.default(data);
        const event = {
            eventId: eventId || uuid_1.default.v4(),
            eventType,
            data
        };
        if (metadata !== undefined) {
            event.metadata = metadata;
        }
        return event;
    }
    close() {
        this.connection.close();
        return this;
    }
}
exports.NestjsEventStore = NestjsEventStore;
//# sourceMappingURL=nestjs-event-store.class.js.map